@@ -1,14 +1,18 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
-from trytond.model import fields
+from trytond import backend
+from trytond.model import ModelSQL, ValueMixin, fields
 from trytond.pyson import Eval
 from trytond.pool import Pool, PoolMeta
 from trytond.transaction import Transaction
-from trytond.tools import grouped_slice
+from trytond.tools.multivalue import migrate_property
 
 
-__all__ = ['PurchaseRequest', 'PurchaseConfig', 'Purchase', 'PurchaseLine',
+__all__ = ['PurchaseRequest', 'PurchaseConfig',
+    'PurchaseConfigPurchaseDropLocation', 'Purchase', 'PurchaseLine',
     'ProductSupplier', 'CreatePurchase', 'PurchaseHandleShipmentException']
+purchase_drop_location = fields.Many2One(
+    'stock.location', "Purchase Drop Location", domain=[('type', '=', 'drop')])
 
 
 class PurchaseRequest:
@@ -32,9 +36,47 @@ class PurchaseConfig:
     __metaclass__ = PoolMeta
     __name__ = 'purchase.configuration'
 
-    purchase_drop_location = fields.Property(
-        fields.Many2One('stock.location', 'Purchase Drop Location',
-            domain=[('type', '=', 'drop')]))
+    purchase_drop_location = fields.MultiValue(purchase_drop_location)
+
+    @classmethod
+    def default_purchase_drop_location(cls, **pattern):
+        return cls.multivalue_model(
+            'purchase_drop_location').default_purchase_drop_location()
+
+
+class PurchaseConfigPurchaseDropLocation(ModelSQL, ValueMixin):
+    "Purchase Configuration Purchase Drop Location"
+    __name__ = 'purchase.configuration.purchase_drop_location'
+    purchase_drop_location = purchase_drop_location
+
+    @classmethod
+    def __register__(cls, module_name):
+        TableHandler = backend.get('TableHandler')
+        exist = TableHandler.table_exist(cls._table)
+
+        super(PurchaseConfigPurchaseDropLocation, cls).__register__(
+            module_name)
+
+        if not exist:
+            cls._migrate_property([], [], [])
+
+    @classmethod
+    def _migrate_property(cls, field_names, value_names, fields):
+        field_names.append('purchase_drop_location')
+        value_names.append('purchase_drop_location')
+        migrate_property(
+            'purchase.configuration', field_names, cls, value_names,
+            fields=fields)
+
+    @classmethod
+    def default_purchase_drop_location(cls):
+        pool = Pool()
+        ModelData = pool.get('ir.model.data')
+        try:
+            return ModelData.get_id(
+                'sale_supply_drop_shipment', 'location_drop')
+        except KeyError:
+            return None
 
 
 class Purchase:
@@ -148,9 +190,18 @@ class ProductSupplier:
 
     drop_shipment = fields.Boolean('Drop Shipment',
         states={
-            'invisible': ~Eval('_parent_product', {}).get('supply_on_sale',
-                False),
-            })
+            'invisible': ~Eval('drop_shipment_available', False),
+            },
+        depends=['drop_shipment_available'])
+    drop_shipment_available = fields.Function(
+        fields.Boolean("Drop Shipment Available"),
+        'on_change_with_drop_shipment_available')
+
+    @fields.depends('product',
+        '_parent_product.type', '_parent_product.supply_on_sale')
+    def on_change_with_drop_shipment_available(self, name=None):
+        if self.product and self.product.type in {'goods', 'assets'}:
+            return self.product.supply_on_sale
 
 
 class CreatePurchase:
@@ -176,9 +227,7 @@ class PurchaseHandleShipmentException:
     def transition_handle(self):
         pool = Pool()
         Sale = pool.get('sale.sale')
-        SaleLine = pool.get('sale.line')
         Purchase = pool.get('purchase.purchase')
-        PurchaseRequest = pool.get('purchase.request')
         Move = pool.get('stock.move')
 
         super(PurchaseHandleShipmentException, self).transition_handle()
@@ -189,29 +238,16 @@ class PurchaseHandleShipmentException:
         domain_moves = set(self.ask.domain_moves)
         purchase = Purchase(Transaction().context['active_id'])
 
-        requests = []
-        for sub_lines in grouped_slice([pl.id for pl in purchase.lines]):
-            requests += PurchaseRequest.search([
-                    ('purchase_line', 'in', list(sub_lines)),
-                    ])
-        pline2request = {r.purchase_line: r for r in requests}
-        request2sline = {}
-        for sub_requests in grouped_slice(requests):
-            sale_lines = SaleLine.search([
-                    ('purchase_request', 'in', [r.id for r in sub_requests]),
-                    ])
-            request2sline.update({sl.purchase_request: sl
-                    for sl in sale_lines})
-
         for line in purchase.lines:
             if not set(line.moves) & domain_moves:
                 continue
             if not any(m in to_recreate for m in line.moves):
-                sale_line = request2sline[pline2request[line]]
-                moves.update({m for m in sale_line.moves
-                        if (m.state != 'done'
-                            and m.from_location.type == 'drop')})
-                sales.add(sale_line.sale)
+                for request in line.requests:
+                    for sale_line in request.sale_lines:
+                        moves.update({m for m in sale_line.moves
+                                if (m.state != 'done'
+                                    and m.from_location.type == 'drop')})
+                        sales.add(sale_line.sale)
 
         if moves:
             Move.cancel(list(moves))
