@@ -1,20 +1,62 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
-from trytond.model import fields
+from trytond import backend
+from trytond.model import ModelSQL, ValueMixin, fields
 from trytond.pyson import Eval
 from trytond.pool import Pool, PoolMeta
 from trytond.transaction import Transaction
+from trytond.tools.multivalue import migrate_property
 
-__all__ = ['SaleConfig', 'Sale', 'SaleLine']
+__all__ = ['SaleConfig', 'SaleConfigSaleDropLocation', 'Sale', 'SaleLine']
+sale_drop_location = fields.Many2One(
+    'stock.location', "Sale Drop Location", domain=[('type', '=', 'drop')])
 
 
 class SaleConfig:
     __metaclass__ = PoolMeta
     __name__ = 'sale.configuration'
 
-    sale_drop_location = fields.Property(
-        fields.Many2One('stock.location', 'Sale Drop Location',
-            domain=[('type', '=', 'drop')]))
+    sale_drop_location = fields.MultiValue(sale_drop_location)
+
+    @classmethod
+    def default_sale_drop_location(cls, **pattern):
+        return cls.multivalue_model(
+            'sale_drop_location').default_sale_drop_location()
+
+
+class SaleConfigSaleDropLocation(ModelSQL, ValueMixin):
+    "Sale Configuration Sale Drop Location"
+    __name__ = 'sale.configuration.sale_drop_location'
+    sale_drop_location = sale_drop_location
+
+    @classmethod
+    def __register__(cls, module_name):
+        TableHandler = backend.get('TableHandler')
+        exist = TableHandler.table_exist(cls._table)
+
+        super(SaleConfigSaleDropLocation, cls).__register__(
+            module_name)
+
+        if not exist:
+            cls._migrate_property([], [], [])
+
+    @classmethod
+    def _migrate_property(cls, field_names, value_names, fields):
+        field_names.append('sale_drop_location')
+        value_names.append('sale_drop_location')
+        migrate_property(
+            'sale.configuration', field_names, cls, value_names,
+            fields=fields)
+
+    @classmethod
+    def default_sale_drop_location(cls):
+        pool = Pool()
+        ModelData = pool.get('ir.model.data')
+        try:
+            return ModelData.get_id(
+                'sale_supply_drop_shipment', 'location_drop')
+        except KeyError:
+            return None
 
 
 class Sale:
