@@ -3,29 +3,86 @@
 from collections import defaultdict
 from decimal import Decimal
 
+from sql import Column
 from sql.operators import Concat
 from sql.aggregate import Count
 
+from trytond import backend
 from trytond.model import Workflow, ModelView, ModelSQL, fields
 from trytond.pyson import Eval, If
 from trytond.pool import Pool, PoolMeta
 from trytond.transaction import Transaction
-from trytond.tools import grouped_slice
+from trytond.tools import grouped_slice, cursor_dict
 
 
-__all__ = ['Configuration', 'ShipmentDrop', 'Move']
+__all__ = ['Configuration', 'ConfigurationSequence', 'ShipmentDrop', 'Move']
 
 
 class Configuration:
     __metaclass__ = PoolMeta
     __name__ = 'stock.configuration'
 
-    shipment_drop_sequence = fields.Property(fields.Many2One('ir.sequence',
-            'Drop Shipment Sequence', domain=[
+    shipment_drop_sequence = fields.MultiValue(fields.Many2One(
+            'ir.sequence', "Drop Shipment Sequence", required=True,
+            domain=[
                 ('company', 'in',
                     [Eval('context', {}).get('company', -1), None]),
                 ('code', '=', 'stock.shipment.drop'),
-                ], required=True))
+                ]))
+
+    @classmethod
+    def multivalue_model(cls, field):
+        pool = Pool()
+        if field == 'shipment_drop_sequence':
+            return pool.get('stock.configuration.sequence')
+        return super(Configuration, cls).multivalue_model(field)
+
+    @classmethod
+    def default_shipment_drop_sequence(cls, **pattern):
+        return cls.multivalue_model(
+            'shipment_drop_sequence').default_shipment_drop_sequence()
+
+
+class ConfigurationSequence:
+    __metaclass__ = PoolMeta
+    __name__ = 'stock.configuration.sequence'
+    shipment_drop_sequence = fields.Many2One(
+        'ir.sequence', "Drop Shipment Sequence", required=True,
+        domain=[
+            ('company', 'in', [Eval('company', -1), None]),
+            ('code', '=', 'stock.shipment.drop'),
+            ],
+        depends=['company'])
+
+    @classmethod
+    def __register__(cls, module_name):
+        TableHandler = backend.get('TableHandler')
+        exist = TableHandler.table_exist(cls._table)
+        if exist:
+            table = TableHandler(cls, module_name)
+            exist &= table.column_exist('shipment_drop_sequence')
+
+        super(ConfigurationSequence, cls).__register__(module_name)
+
+        if not exist:
+            cls._migrate_property([], [], [])
+
+    @classmethod
+    def _migrate_property(cls, field_names, value_names, fields):
+        field_names.append('shipment_drop_sequence')
+        value_names.append('shipment_drop_sequence')
+        super(ConfigurationSequence, cls)._migrate_property(
+            field_names, value_names, fields)
+
+    @classmethod
+    def default_shipment_drop_sequence(cls):
+        pool = Pool()
+        ModelData = pool.get('ir.model.data')
+        try:
+            return ModelData.get_id(
+                'sale_supply_drop_shipment', 'sequence_shipment_drop')
+        except KeyError:
+            return None
 
 
 class ShipmentDrop(Workflow, ModelSQL, ModelView):
@@ -160,12 +217,19 @@ class ShipmentDrop(Workflow, ModelSQL, ModelView):
                         values=[drop_shipment_location],
                         where=move.id.in_(sub_ids)))
 
-            create_move = move.insert(values=move.select(
-                    where=move.shipment.like('stock.shipment.drop,%')))
-            cursor.execute(*create_move)
+            cursor.execute(*move.select(limit=1))
+            moves = list(cursor_dict(cursor))
+            if moves:
+                move_columns = moves[0].keys()
+                columns = [Column(move, c) for c in move_columns if c != 'id']
+                create_move = move.insert(
+                    columns=columns, values=move.select(
+                        *columns,
+                        where=move.shipment.like('stock.shipment.drop,%')))
+                cursor.execute(*create_move)
 
             for move_id, customer_location, line_id in move_sales:
-                cursor.execute(move.update(
+                cursor.execute(*move.update(
                         columns=[move.origin, move.from_location,
                             move.to_location],
                         values=[Concat('sale.line,', str(line_id)),
@@ -244,12 +308,15 @@ class ShipmentDrop(Workflow, ModelSQL, ModelView):
         Set planned date of moves for the shipments
         '''
         Move = Pool().get('stock.move')
+        to_write = []
         for shipment in shipments:
             planned_date = shipment._get_move_planned_date()
-            Move.write([m for m in shipment.moves
-                    if m.state not in ('assigned', 'done', 'cancel')], {
-                    'planned_date': planned_date,
-                    })
+            to_write.extend(([m for m in shipment.moves
+                        if m.state not in ('assigned', 'done', 'cancel')], {
+                        'planned_date': planned_date,
+                        }))
+        if to_write:
+            Move.write(*to_write)
 
     def get_moves(self, name):
         if name == 'supplier_moves':
